#include "board.h"

//Implementation ============================================================
/* ==========================================================================
 * Function:
 * 	Initiate Board
 * Description:
 *	Place all unit on board, empty unit are 0
 * ==========================================================================
 */
void initBoard() {
	//place pawn ============================================================
	int i;
	for (i = 0; i <= maxBoard; ++i) {
		board[i][1].piece = PAWN;
		board[i][1].color = BLACK;
		board[i][maxBoard-1].piece = PAWN;
		board[i][maxBoard-1].color = WHITE;
	}
	//=====================================================================
	
	//place rook ==========================================================
	board[0][0].piece = ROOK;
	board[0][0].color = BLACK;
	board[maxBoard][0].piece = ROOK;
	board[maxBoard][0].color = BLACK;
	board[0][maxBoard].piece = ROOK;
	board[0][maxBoard].color = WHITE;
	board[maxBoard][maxBoard].piece = ROOK;
	board[maxBoard][maxBoard].color = WHITE;
	//======================================================================

	//place knight ==========================================================
	board[1][0].piece = KNIGHT;
	board[1][0].color = BLACK;
	board[maxBoard-1][0].piece = KNIGHT;
	board[maxBoard-1][0].color = BLACK;
	board[1][maxBoard].piece = KNIGHT;
	board[1][maxBoard].color = WHITE;
	board[maxBoard-1][maxBoard].piece = KNIGHT;
	board[maxBoard-1][maxBoard].color = WHITE;
	//==========================================================================

	//place bishop ==========================================================
	board[2][0].piece = BISHOP;
	board[2][0].color = BLACK;
	board[maxBoard-2][0].piece = BISHOP;
	board[maxBoard-2][0].color = BLACK;
	board[2][maxBoard].piece = BISHOP;
	board[2][maxBoard].color = WHITE;
	board[maxBoard-2][maxBoard].piece = BISHOP;
	board[maxBoard-2][maxBoard].color = WHITE;
	//==========================================================================

	//place queen ==============================================================
	board[3][0].piece = QUEEN;
	board[3][0].color = BLACK;
	board[3][maxBoard].piece = QUEEN;
	board[3][maxBoard].color = WHITE;
	//==========================================================================

	//place king ==============================================================
	board[4][0].piece = KING;
	board[4][0].color =  BLACK;
	board[4][maxBoard].piece = KING;
	board[4][maxBoard].color = WHITE;
	//==========================================================================
}
//==============================================================================
/*=============================================================================
 * function:
 * 	Move piece
 * Description:
 * 	Get new place for unit and move the unit
 * 	If the move is not valid return 1, and print the reason why
 * ============================================================================
 */
int move (int old[2], int new[2]) {
	//variable ======================================
	int status = 0;
	PIECE piece = board[old[X]][old[Y]].piece;
	COLOR color = board[old[X]][old[Y]].color;
	int flag = board[old[X]][old[Y]].flag;
	//==============================================

	//perform move by piece=========================
	switch (piece) {
		case PAWN:
			status = pawnMove(color, flag, old, new);
			break;
		case ROOK:
			status = rookMove(color, flag, old, new);
			break;
		case KNIGHT:
			status = knightMove(color, flag, old, new);
			break;
		case BISHOP:
			status = bishopMove(color, flag, old, new);
			break;
		case QUEEN:
			status = queenMove(color, flag, old, new);
			break;
		case KING:
			status = kingMove(color, flag, old, new);
			break;
		default:
			status = 1;
			break;
	}
	//case of illegal move =============================
	//if (status != 0) {
	//	performStatus(status);
	//}
	//==================================================
	return status;
}
//===================================================================================


// PAWN Moves ***********************************************************************
/*===================================================================================
 * Function:
 * 	Move pawn
 * Description:
 * 	Move the pawn case of legal move
 * 	case of illegal move return reason
 * 	Return
 * 	 0	all OK
 * 	 1	out of board
 * 	 2	go back
 * 	 3	same place
 * 	 4	long step
 * 	 5	new place is on your unit
 * 	 6	not straight move
 * 	 7	not in start position
 * 	 8	two step on enemy place
 * 	 9	in x more then 1
 * 	 10	on enemy place
 * 	 11 need to be enemy to eat
 * 	 100 kill king
 *===================================================================================
 */
int pawnMove (COLOR color,int flag, int old[2], int new[2]) {
	if (outOfBoard(new) == 1) {
		return 1;
	} else if  (goBack(color, old[Y], new[Y]) == 1) {
		return 2;
	} else if (samePlace(old, new) == 1) {
		return 3;
	} else if (longStep(color, old[Y], new[Y]) == 1) {
		return 4;
	} else if (yourUnit(old, new) == 1) {
		return 5;
	} else if (twoStep(color, old, new) == 1) {
		return 6;
	} else if (twoStep(color, old, new) == 2) {
		return 7;
	} else if (twoStep(color, old, new) == 3) {
		return 8;
	} else if (oneStep(color, old, new) == 1) {
		return 9;
	} else if (oneStep(color, old, new) == 2) {
		return 10;
	} else if (oneStep(color, old, new) == 3) {
		return 11;
	}
	return performMove(old, new);
}
// ==================================================================================



/*===================================================================================
 * Function:
 * 	Go Back
 * Description:
 * 	Check if new place is not back
 * 	Return
 * 	 0 All OK
 * 	 1 The unit go back
 *===================================================================================
 */
int goBack (COLOR color, int old, int new) {
	if (color == WHITE) {
		if (new > old) {
			return 1;
		}
	} else if (new < old) {
		return 1;
	}
	return 0;
}

/*===================================================================================
 * Function:
 * 	Long Move
 * Description:
 * 	Check if the new place bigger than 2 step in X
 * 	Return
 * 	 0 All OK
 * 	 1 Long Step
 *===================================================================================
 */
int longStep (COLOR color,int old, int new) {
	if (color == WHITE) {
		if (old - new > 2) {
			return 1;
		}
	} else if (new - old > 2){
		return 1;
	}
	return 0;
}

/*===================================================================================
 * Function:
 * 	Two Step
 * Description:
 * 	Check if two step is legal
 * 	Return
 * 	 0 All OK
 * 	 1 New Place is not straight
 * 	 2 The PAWN not in start place
 * 	 3 The new place on enemy unit
 *===================================================================================
 */
int twoStep (COLOR color, int old[2], int new[2]) {
	if (color == WHITE) {
		if ((old[Y] - new[Y]) == 2) {
			if ((old[X] - new[X]) != 0) {
				return 1;
			} else if (old[Y] != 6) {
				return 2;
			} else if (enemyUnit(old, new) == 1) {
				return 3;
			}
		}
	}
	if (color == BLACK) {
		if ((new[Y] - old[Y]) == 2) {
			if ((old[X] - new[X]) != 0) {
				return 1;
			} else if (old[Y] != 1) {
				return 2;
			} else if (enemyUnit(old, new) == 1) {
				return 3;
			}
		}
	}
	return 0;
}

/*===================================================================================
 * Function:
 * 	One Step
 * Description:
 * 	Check one step illegal moves
 * Return
 *  0 All OK
 *  1 The X new place bigger than one
 *  2 The new place is straight but on enemy unit
 *  3 Eat action but no enemy unit in the new place
 *===================================================================================
 */
int oneStep (COLOR color, int old[2], int new[2]) {
	if ((new[X] - old[X] > 1) || (new[X]-old[X] < -1)) {
		return 1;
	} else if ((enemyUnit(old, new) == 1) && (new[X] - old[X] == 0)) {
		return 2;
	} else if ((enemyUnit(old, new) == 0) && ((new[X] - old[X] == 1) || (new[X] - old[X] == -1))) {
		return 3;
	}
	return 0;
}

/*===================================================================================
 * Function:
 * 	PAWN became QUENN
 * Description:
 * 	The new place is in the end of the board and the PAWN became QUENN
 * 	Return
 * 	 0 Not end of the board
 * 	 1 End of the board
 *===================================================================================
 */
int pawnBecameQueen (COLOR color, int new) {
	if (color == WHITE) {
		if (new == 0) {
			return 1;
		}
	} else if (new == 7) {
			return 1;
	}
	return 0;
}
//====================================================================================
//************************************************************************************

// ROOK moves ************************************************************************
/*===================================================================================
 * Function:
 * 	Move rook
 * Description:
 * 	Move the pawn case of legal move
 * 	case of illegal move return reason
 * 	Return
 * 	 0	all OK
 * 	 1	out of board
 * 	 2 same place
 * 	 3 not straight move
 * 	 4 unit in the way
 * 	 100 kill king
 *===================================================================================
 */
int rookMove (COLOR color, int flag, int old[2], int new[2]) {
	if (outOfBoard(new) == 1) {
		return 1;
	} else if (samePlace(old, new) == 1) {
		return 2;
	} else if (notStraightMove(old, new) == 1) {
		return 3;
	} else if (unitInTheWayStraight(old, new) == 1) {
		return 4;
	} else if (yourUnit(old, new) == 1) {
		return 5;
	}
	return performMove(old, new);
}
//==============================================================================
int notStraightMove(int old[2], int new[2]) {
	if (!(new[X]-old[X] != 0 || new[Y] - old[Y] != 0)) {
		return 1;
	} else if ((new[X]-old[X] != 0 && new[Y] - old[Y] != 0)) {
		return 1;
	}
	return 0;
}
//=====================================================================================
int unitInTheWayStraight(int old[2], int new[2]) {
	int i;
	int inTheWay[2];
	if (new[X] - old[X] == 0) {
		if (new[Y] - old[Y] > 0) {
			for (i = old[Y]+1; i < new[Y]; ++i) {
				inTheWay[X] = old[X];
				inTheWay[Y] = i;
				if (yourUnit(old, inTheWay) == 1 || enemyUnit(old, inTheWay) == 1) {
					return 1;
				}
			}
		} else {
			for (i = old[Y]-1; i > new[Y]; --i) {
				inTheWay[X] = old[X];
				inTheWay[Y] = i;
				if (yourUnit(old, inTheWay) == 1 || enemyUnit(old, inTheWay) == 1) {
					return 1;
				}
			}
		}
	}
	if (new[Y] - old[Y] == 0) {
		if (new[X] - old[X] > 0) {
			for (i = old[X]+1; i < new[X]; ++i) {
				inTheWay[X] = i;
				inTheWay[Y] = old[Y];
				if (yourUnit(old, inTheWay) == 1 || enemyUnit(old, inTheWay) == 1) {
					return 1;
				}
			}
		} else {
			for (i = old[X]-1; i > new[X]; --i) {
				inTheWay[X] = i;
				inTheWay[Y] = old[Y];
				if (yourUnit(old, inTheWay) == 1 || enemyUnit(old, inTheWay) == 1) {
					return 1;
				}
			}
		}
	}
	return 0;
}
// =====================================================================================

// Knight moves ************************************************************************
/*===================================================================================
 * Function:
 * 	Move knight
 * Description:
 * 	Move the knight case of legal move
 * 	case of illegal move return reason
 * 	Return
 * 	 0	 all OK
 * 	 1	 out of board
 * 	 2   same place
 * 	 3   not correct move
 * 	 100 kill king
 *===================================================================================
 */
int knightMove (COLOR color, int flag, int old[2], int new[2]) {
	if (outOfBoard(new) == 1) {
			return 1;
		} else if (samePlace(old, new) == 1) {
			return 2;
		} else if (notLegalKnightlMove(old, new) == 1) {
			return 3;
		}
	return performMove(old, new);
}

/*===================================================================================
 * Function:
 * 	not legal move for knight
 * Description:
 * 	Check if the move is legal for knight
 * 	Return
 * 	 0 legal move
 * 	 1 not legal move
 *===================================================================================*/
int notLegalKnightlMove(int old[2], int new[2]) {
	if ((new[0] - old[0] == 2) && (new[1]) - old[1] == 1) {
		return 0;
	} else if ((new[0] - old[0] == 2) && (new[1]) - old[1] == -1) {
		return 0;
	} else if ((new[0] - old[0] == -2) && (new[1]) - old[1] == 1) {
		return 0;
	} else if ((new[0] - old[0] == -2) && (new[1]) - old[1] == -1) {
		return 0;
	} else if ((new[0] - old[0] == 1) && (new[1]) - old[1] == 2) {
		return 0;
	} else if ((new[0] - old[0] == 1) && (new[1]) - old[1] == -2) {
		return 0;
	} else if ((new[0] - old[0] == -1) && (new[1]) - old[1] == 2) {
		return 0;
	} else if ((new[0] - old[0] == -1) && (new[1]) - old[1] == -2) {
		return 0;
	}
	return 1;
}

// Bishop moves ************************************************************************
/*===================================================================================
 * Function:
 * 	Move bishop
 * Description:
 * 	Move the bishop case of legal move
 * 	case of illegal move return reason
 * 	Return
 * 	 0	all OK
 * 	 1	out of board
 * 	 2 same place
 * 	 3 not diagonal move
 * 	 4 unit in the way
 * 	 5 you unit
 * 	 100 kill king
 *===================================================================================
 */
int bishopMove (COLOR color, int flag, int old[2], int new[2]) {
	if (outOfBoard(new) == 1) {
		return 1;
	} else if (samePlace(old, new) == 1) {
		return 2;
	} else if (notDiagonalMove(old, new) == 1) {
		return 3;
	} else if (unitInTheWayDiagonal(old, new) == 1) {
		return 4;
	} else if (yourUnit(old, new) == 1) {
		return 5;
	}
	return performMove(old, new);
}
// =====================================================================================

/*===================================================================================
 * Function:
 * 	Not diagonal move
 * Description:
 * 	Check if the move is diagonal
 * 	Return
 * 	 0 diagonal move
 * 	 1 not diagonal move
 *===================================================================================
*/
int notDiagonalMove (int old[2], int new[2]) {
	int x = new[X] - old[X];
	int y = new[Y] - old[Y];
	if (x < 0) {
		x = x * -1;
	}
	if (y < 0) {
			y = y * -1;
	}
	if (x != y) {
		return 1;
	}
	return 0;
}

/*===================================================================================
 * Function:
 * 	Unit in the way for diagonal move
 * Description:
 * 	Check if any unit in the way
 * 	Return
 * 	 0 no unit in the way
 * 	 1 unit in the way
 *===================================================================================
*/
int unitInTheWayDiagonal (int old[2], int new[2]) {
	int current[2];
	current[X] = old[X];
	current[Y] = old[Y];
	while (current[X] != new[X]) {
		if (new[X] > current[X]) {
			current[X]++;
		} else {
			current[X]--;
		}
		if (current[X] == new[X]) {
			return 0;
		}
		if (new[Y] > current[Y]) {
			current[Y]++;
		} else {
			current[Y]--;
		}
		if (yourUnit(old, current) == 1 || enemyUnit(old, current) == 1) {
				return 1;
		}
	}
	return 0;
}

/*===================================================================================
 * Function:
 * 	Move queen
 * Description:
 * 	Move the queen case of legal move
 * 	case of illegal move return reason
 * 	Return
 * 	 0	all OK
 * 	 1	out of board
 * 	 2 same place
 * 	 3 not legal move
 * 	 4 unit in your way
 * 	 5 your unit
 * 	 100 kill king
 *===================================================================================
 */
int queenMove (COLOR color, int flag, int old[2], int new[2]) {
	if (outOfBoard(new) == 1) {
		return 1;
	} else if (samePlace(old, new) == 1) {
		return 2;
	} else if (notLegalMoveQueen(old, new) == 0) {
		return 3;
	} else if (notLegalMoveQueen(old, new) == 1) {
		if (unitInTheWayStraight(old, new) == 1) {
			return 4;
		}
	} else if (notLegalMoveQueen(old, new) == 2) {
		if (unitInTheWayDiagonal(old, new) == 1) {
			return 4;
		}
	} else if (yourUnit(old, new) == 1) {
		return 5;
	}
	return performMove(old, new);
}

/*===================================================================================
 * Function:
 * 	not legal move for queen
 * Description:
 * 	Check if the move is legal for queen
 * 	Return
 * 	 0 not legal move
 * 	 1 legal move straight
 * 	 2 legal move diagonal
 *===================================================================================*/
int notLegalMoveQueen (int old[2], int new[2]) {
	int goodMove = 0;
	if (notStraightMove(old, new) == 1) {
		goodMove=1;
	} else if (notDiagonalMove(old, new) == 1) {
		goodMove = 2;
	}
	return goodMove;
}

/*===================================================================================
 * Function:
 * 	Move king
 * Description:
 * 	Move the king case of legal move
 * 	case of illegal move return reason
 * 	Return
 * 	 0	all OK
 * 	 1	out of board
 * 	 2 same place
 * 	 3 not legal move
 * 	 5 your unit
 * 	 100 kill king
 *===================================================================================
 */
int kingMove (COLOR color, int flag, int old[2], int new[2]) {
	if (outOfBoard(new) == 1) {
		return 1;
	} else if (samePlace(old, new) == 1) {
		return 2;
	} else if (notLegalMoveKing(old, new) == 1) {
		return 3;
	} else if (yourUnit(old, new) == 1) {
		return 5;
	}
	return performMove(old, new);
}

/*===================================================================================
 * Function:
 * 	not legal move for queen
 * Description:
 * 	Check if the move is legal for queen
 * 	Return
 * 	 0 not legal move
 * 	 1 not legal move
 *===================================================================================*/
int notLegalMoveKing(int old[2],int new[2]) {
	if ((new[X]-old[X] != 1) || (new[X]-old[X] != -1) || (new[1]-old[Y] != 1) || (new[Y]-old[Y] != -1)) {
		return 1;
	}
	return 0;
}


//===================================================================================
// Functions to all Units ===========================================================
//===================================================================================

/*===================================================================================
 * Function:
 * 	Out of Board
 * Description:
 * 	Check if new place in the board limit
 * 	Return
 * 	 0 All OK
 *   1 Out of board
 *===================================================================================
 */
int outOfBoard (int new[2]) {
	if (new[X] < 0 || new[X] > maxBoard || new[Y] < 0 || new[Y] > maxBoard) {
		return 1;
	}
	return 0;
}

/*===================================================================================
 * Function:
 * 	Your Unit
 * Description:
 * 	New place on your unit
 * 	Return
 * 	 0 All OK
 * 	 1 New place on your unit
 *===================================================================================
 */
int yourUnit (int old[2], int new[2]) {
	if (board[old[X]][old[Y]].color == board[new[X]][new[Y]].color) {
		return 1;
	}
	return 0;
}

/*===================================================================================
* Function:
 * 	Enemy Unit
 * Description:
 * 	New place on enemy unit
 * Return
 *  0 All OK
 *  1 New place on enemy unit
 *===================================================================================
 */
int enemyUnit (int old[2], int new[2]) {
	COLOR color;
	color = board[old[X]][old[Y]].color;
	if (color == WHITE) {
		if (board[new[X]][new[Y]].color == BLACK) {
			return 1;
		}
	} else {
		if (board[new[X]][new[Y]].color == WHITE) {
			return 1;
		}
	}
	return 0;
}

/*===================================================================================
 * Function:
 * 	Same Place
 * Description:
 * 	New place equal to Old place
 * Return
 *  0 All OK
 *  1 New place equal to old place
 *===================================================================================
 */
int samePlace(int old[2], int new[2]) {
	if ((old[X] == new[X]) && (old[Y] == new[Y])) {
		return 1;
	}
	return 0;
}
int killKing(int new[2]) {
	if (board[new[X]][new[Y]].piece == KING) {
		return 100;
	}
	return 0;
}

/*===================================================================================
 * Function:
 * 	Perform move
 * Description:
 * 	Perform the move put the unit in the new place and put 0 in the old place
 * Return
 *  0   Move perform, king not kill
 *  100 Move perform, king kill
 *===================================================================================*/
int performMove (int old[2], int new[2]) {
	int king=0;
	king = killKing(new);
	board[new[X]][new[Y]] = board[old[X]][old[Y]];
	board[old[X]][old[Y]].color=0;
	board[old[X]][old[Y]].piece=0;
	board[old[X]][old[Y]].flag=0;
	return king;
}
//===================================================================================
//===================================================================================


int performStatus(int status) {
	return 0;
}
